@@ -4,9 +4,10 @@ import logging
 import click
 import json
 
-from ..workspace import Workspace
-from .. import modifiers
-from .. import utils
+from pyhf.workspace import Workspace
+from pyhf import modifiers
+from pyhf import parameters
+from pyhf import utils
 
 log = logging.getLogger(__name__)
 
@@ -32,7 +33,7 @@ def inspect(workspace, output_file, measurement):
 
     .. code-block:: shell
 
-        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/master/docs/examples/json/2-bin_1-channel.json | pyhf inspect
+        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/main/docs/examples/json/2-bin_1-channel.json | pyhf inspect
                   Summary
             ------------------
                channels  1
@@ -59,7 +60,7 @@ def inspect(workspace, output_file, measurement):
         (*) Measurement            mu            (none)
 
     """
-    with click.open_file(workspace, 'r') as specstream:
+    with click.open_file(workspace, "r", encoding="utf-8") as specstream:
         spec = json.load(specstream)
 
     ws = Workspace(spec)
@@ -72,14 +73,17 @@ def inspect(workspace, output_file, measurement):
     ]
     result['modifiers'] = dict(ws.modifiers)
 
+    parset_descr = {
+        parameters.paramsets.unconstrained: 'unconstrained',
+        parameters.paramsets.constrained_by_normal: 'constrained_by_normal',
+        parameters.paramsets.constrained_by_poisson: 'constrained_by_poisson',
+    }
+
+    model = ws.model()
+
     result['parameters'] = sorted(
-        (
-            parname,
-            modifiers.registry[result['modifiers'][parname]]
-            .required_parset([], [])['paramset_type']
-            .__name__,
-        )
-        for parname in ws.parameters
+        (parset_name, parset_descr[type(parset_spec['paramset'])])
+        for parset_name, parset_spec in model.config.par_map.items()
     )
     result['systematics'] = [
         (
@@ -97,7 +101,7 @@ def inspect(workspace, output_file, measurement):
 
     maxlen_channels = max(map(len, ws.channels))
     maxlen_samples = max(map(len, ws.samples))
-    maxlen_parameters = max(map(len, ws.parameters))
+    maxlen_parameters = max(map(len, [p for p, _ in result['parameters']]))
     maxlen_measurements = max(map(lambda x: len(x[0]), result['measurements']))
     maxlen = max(
         [maxlen_channels, maxlen_samples, maxlen_parameters, maxlen_measurements]
@@ -154,7 +158,7 @@ def inspect(workspace, output_file, measurement):
     click.echo()
 
     if output_file:
-        with open(output_file, 'w+') as out_file:
+        with open(output_file, "w+", encoding="utf-8") as out_file:
             json.dump(result, out_file, indent=4, sort_keys=True)
         log.debug(f"Written to {output_file:s}")
 
@@ -174,7 +178,7 @@ def inspect(workspace, output_file, measurement):
     '--modifier-type',
     default=[],
     multiple=True,
-    type=click.Choice(modifiers.uncombined.keys()),
+    type=click.Choice(modifiers.histfactory_set.keys()),
 )
 @click.option('--measurement', default=[], multiple=True, metavar='<MEASUREMENT>...')
 def prune(
@@ -185,7 +189,7 @@ def prune(
 
     See :func:`pyhf.workspace.Workspace.prune` for more information.
     """
-    with click.open_file(workspace, 'r') as specstream:
+    with click.open_file(workspace, "r", encoding="utf-8") as specstream:
         spec = json.load(specstream)
 
     ws = Workspace(spec)
@@ -200,7 +204,7 @@ def prune(
     if output_file is None:
         click.echo(json.dumps(pruned_ws, indent=4, sort_keys=True))
     else:
-        with open(output_file, 'w+') as out_file:
+        with open(output_file, "w+", encoding="utf-8") as out_file:
             json.dump(pruned_ws, out_file, indent=4, sort_keys=True)
         log.debug(f"Written to {output_file:s}")
 
@@ -249,7 +253,7 @@ def rename(workspace, output_file, channel, sample, modifier, measurement):
 
     See :func:`pyhf.workspace.Workspace.rename` for more information.
     """
-    with click.open_file(workspace, 'r') as specstream:
+    with click.open_file(workspace, "r", encoding="utf-8") as specstream:
         spec = json.load(specstream)
 
     ws = Workspace(spec)
@@ -263,7 +267,7 @@ def rename(workspace, output_file, channel, sample, modifier, measurement):
     if output_file is None:
         click.echo(json.dumps(renamed_ws, indent=4, sort_keys=True))
     else:
-        with open(output_file, 'w+') as out_file:
+        with open(output_file, "w+", encoding="utf-8") as out_file:
             json.dump(renamed_ws, out_file, indent=4, sort_keys=True)
         log.debug(f"Written to {output_file:s}")
 
@@ -294,10 +298,10 @@ def combine(workspace_one, workspace_two, join, output_file, merge_channels):
 
     See :func:`pyhf.workspace.Workspace.combine` for more information.
     """
-    with click.open_file(workspace_one, 'r') as specstream:
+    with click.open_file(workspace_one, "r", encoding="utf-8") as specstream:
         spec_one = json.load(specstream)
 
-    with click.open_file(workspace_two, 'r') as specstream:
+    with click.open_file(workspace_two, "r", encoding="utf-8") as specstream:
         spec_two = json.load(specstream)
 
     ws_one = Workspace(spec_one)
@@ -309,7 +313,7 @@ def combine(workspace_one, workspace_two, join, output_file, merge_channels):
     if output_file is None:
         click.echo(json.dumps(combined_ws, indent=4, sort_keys=True))
     else:
-        with open(output_file, 'w+') as out_file:
+        with open(output_file, "w+", encoding="utf-8") as out_file:
             json.dump(combined_ws, out_file, indent=4, sort_keys=True)
         log.debug(f"Written to {output_file:s}")
 
@@ -340,10 +344,10 @@ def digest(workspace, algorithm, output_json):
 
     .. code-block:: shell
 
-        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/master/docs/examples/json/2-bin_1-channel.json | pyhf digest
+        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/main/docs/examples/json/2-bin_1-channel.json | pyhf digest
         sha256:dad8822af55205d60152cbe4303929042dbd9d4839012e055e7c6b6459d68d73
     """
-    with click.open_file(workspace, 'r') as specstream:
+    with click.open_file(workspace, "r", encoding="utf-8") as specstream:
         spec = json.load(specstream)
 
     workspace = Workspace(spec)
@@ -379,17 +383,17 @@ def sort(workspace, output_file):
 
     .. code-block:: shell
 
-        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/master/docs/examples/json/2-bin_1-channel.json | pyhf sort | jq '.' | md5
+        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/main/docs/examples/json/2-bin_1-channel.json | pyhf sort | jq '.' | md5
         8be5186ec249d2704e14dd29ef05ffb0
 
     .. code-block:: shell
 
-        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/master/docs/examples/json/2-bin_1-channel.json | jq -S '.channels|=sort_by(.name)|.channels[].samples|=sort_by(.name)|.channels[].samples[].modifiers|=sort_by(.name,.type)|.observations|=sort_by(.name)' | md5
+        $ curl -sL https://raw.githubusercontent.com/scikit-hep/pyhf/main/docs/examples/json/2-bin_1-channel.json | jq -S '.channels|=sort_by(.name)|.channels[].samples|=sort_by(.name)|.channels[].samples[].modifiers|=sort_by(.name,.type)|.observations|=sort_by(.name)' | md5
         8be5186ec249d2704e14dd29ef05ffb0
 
 
     """
-    with click.open_file(workspace, 'r') as specstream:
+    with click.open_file(workspace, "r", encoding="utf-8") as specstream:
         spec = json.load(specstream)
 
     workspace = Workspace(spec)
@@ -398,6 +402,6 @@ def sort(workspace, output_file):
     if output_file is None:
         click.echo(json.dumps(sorted_ws, indent=4, sort_keys=True))
     else:
-        with open(output_file, 'w+') as out_file:
+        with open(output_file, "w+", encoding="utf-8") as out_file:
             json.dump(sorted_ws, out_file, indent=4, sort_keys=True)
         log.debug(f"Written to {output_file}")
