@@ -1,6 +1,6 @@
 """Module for Maximum Likelihood Estimation."""
-from .. import get_backend
-from ..exceptions import UnspecifiedPOI
+from pyhf import get_backend
+from pyhf.exceptions import UnspecifiedPOI
 
 __all__ = ["fit", "fixed_poi_fit", "twice_nll"]
 
@@ -12,7 +12,7 @@ def __dir__():
 def twice_nll(pars, data, pdf):
     r"""
     Two times the negative log-likelihood of the model parameters, :math:`\left(\mu, \boldsymbol{\theta}\right)`, given the observed data.
-    It is used in the calculation of the test statistic, :math:`t_{\mu}`, as defiend in Equation (8) in :xref:`arXiv:1007.1727`
+    It is used in the calculation of the test statistic, :math:`t_{\mu}`, as defined in Equation (8) in :xref:`arXiv:1007.1727`
 
     .. math::
 
