@@ -1,34 +1,67 @@
 import logging
 
-from . import modifier
-from .. import get_backend, events
-from ..parameters import unconstrained, ParamViewer
+from pyhf import get_backend, events
+from pyhf.parameters import ParamViewer
 
 log = logging.getLogger(__name__)
 
 
-@modifier(name='normfactor', op_code='multiplication')
-class normfactor:
-    @classmethod
-    def required_parset(cls, sample_data, modifier_data):
-        return {
-            'paramset_type': unconstrained,
-            'n_parameters': 1,
-            'is_constrained': cls.is_constrained,
-            'is_shared': True,
-            'is_scalar': True,
-            'inits': (1.0,),
-            'bounds': ((0, 10),),
-            'fixed': False,
-        }
+def required_parset(sample_data, modifier_data):
+    return {
+        'paramset_type': 'unconstrained',
+        'n_parameters': 1,
+        'is_scalar': True,
+        'inits': (1.0,),
+        'bounds': ((0, 10),),
+        'fixed': False,
+    }
+
+
+class normfactor_builder:
+    """Builder class for collecting normfactor modifier data"""
+
+    is_shared = True
+
+    def __init__(self, config):
+        self.builder_data = {}
+        self.config = config
+        self.required_parsets = {}
+
+    def collect(self, thismod, nom):
+        maskval = True if thismod else False
+        mask = [maskval] * len(nom)
+        return {'mask': mask}
+
+    def append(self, key, channel, sample, thismod, defined_samp):
+        self.builder_data.setdefault(key, {}).setdefault(sample, {}).setdefault(
+            'data', {'mask': []}
+        )
+        nom = (
+            defined_samp['data']
+            if defined_samp
+            else [0.0] * self.config.channel_nbins[channel]
+        )
+        moddata = self.collect(thismod, nom)
+        self.builder_data[key][sample]['data']['mask'] += moddata['mask']
+        if thismod:
+            self.required_parsets.setdefault(
+                thismod['name'],
+                [required_parset(defined_samp['data'], thismod['data'])],
+            )
+
+    def finalize(self):
+        return self.builder_data
 
 
 class normfactor_combined:
-    def __init__(self, normfactor_mods, pdfconfig, mega_mods, batch_size=None):
+    name = 'normfactor'
+    op_code = 'multiplication'
+
+    def __init__(self, modifiers, pdfconfig, builder_data, batch_size=None):
         self.batch_size = batch_size
 
-        keys = [f'{mtype}/{m}' for m, mtype in normfactor_mods]
-        normfactor_mods = [m for m, _ in normfactor_mods]
+        keys = [f'{mtype}/{m}' for m, mtype in modifiers]
+        normfactor_mods = [m for m, _ in modifiers]
 
         parfield_shape = (
             (self.batch_size, pdfconfig.npars)
@@ -40,7 +73,8 @@ class normfactor_combined:
         )
 
         self._normfactor_mask = [
-            [[mega_mods[m][s]['data']['mask']] for s in pdfconfig.samples] for m in keys
+            [[builder_data[m][s]['data']['mask']] for s in pdfconfig.samples]
+            for m in keys
         ]
         self._precompute()
         events.subscribe('tensorlib_changed')(self._precompute)
