@@ -2,17 +2,20 @@
 
 import copy
 import logging
+from typing import List, Union
 
-from . import get_backend, default_backend
-from . import exceptions
-from . import modifiers
-from . import utils
-from . import events
-from . import probability as prob
-from .constraints import gaussian_constraint_combined, poisson_constraint_combined
-from .parameters import reduce_paramsets_requirements, ParamViewer
-from .tensor.common import _TensorViewer, _tensorviewer_from_sizes
-from .mixins import _ChannelSummaryMixin
+import pyhf.parameters
+import pyhf
+from pyhf.tensor.manager import get_backend
+from pyhf import exceptions
+from pyhf import schema
+from pyhf import events
+from pyhf import probability as prob
+from pyhf.constraints import gaussian_constraint_combined, poisson_constraint_combined
+from pyhf.parameters import reduce_paramsets_requirements, ParamViewer
+from pyhf.tensor.common import _TensorViewer, _tensorviewer_from_sizes
+from pyhf.mixins import _ChannelSummaryMixin
+from pyhf.modifiers import histfactory_set
 
 log = logging.getLogger(__name__)
 
@@ -23,88 +26,76 @@ def __dir__():
     return __all__
 
 
-def _paramset_requirements_from_channelspec(spec, channel_nbins):
-    # bookkeep all requirements for paramsets we need to build
-    _paramsets_requirements = {}
-    # need to keep track in which order we added the constraints
-    # so that we can generate correctly-ordered data
-    for channel in spec['channels']:
-        for sample in channel['samples']:
-            if len(sample['data']) != channel_nbins[channel['name']]:
-                raise exceptions.InvalidModel(
-                    f"The sample {sample['name']:s} has {len(sample['data']):d} bins, but the channel it belongs to ({channel['name']:s}) has {channel_nbins[channel['name']]:d} bins."
-                )
-            for modifier_def in sample['modifiers']:
-                # get the paramset requirements for the given modifier. If
-                # modifier does not exist, we'll have a KeyError
-                try:
-                    paramset_requirements = modifiers.registry[
-                        modifier_def['type']
-                    ].required_parset(sample['data'], modifier_def['data'])
-                except KeyError:
-                    log.exception(
-                        f"Modifier not implemented yet (processing {modifier_def['type']:s}). Available modifiers: {modifiers.registry.keys()}"
-                    )
-                    raise exceptions.InvalidModifier()
-
-                # check the shareability (e.g. for shapesys for example)
-                is_shared = paramset_requirements['is_shared']
-                if not (is_shared) and modifier_def['name'] in _paramsets_requirements:
-                    raise ValueError(
-                        "Trying to add unshared-paramset but other paramsets exist with the same name."
-                    )
-                if is_shared and not (
-                    _paramsets_requirements.get(
-                        modifier_def['name'], [{'is_shared': True}]
-                    )[0]['is_shared']
-                ):
-                    raise ValueError(
-                        "Trying to add shared-paramset but other paramset of same name is indicated to be unshared."
-                    )
-                _paramsets_requirements.setdefault(modifier_def['name'], []).append(
-                    paramset_requirements
-                )
-    return _paramsets_requirements
-
-
-def _paramset_requirements_from_modelspec(spec, channel_nbins):
-    _paramsets_requirements = _paramset_requirements_from_channelspec(
-        spec, channel_nbins
-    )
-
+def _finalize_parameters_specs(user_parameters, _paramsets_requirements):
     # build up a dictionary of the parameter configurations provided by the user
     _paramsets_user_configs = {}
-    for parameter in spec.get('parameters', []):
+    for parameter in user_parameters:
         if parameter['name'] in _paramsets_user_configs:
             raise exceptions.InvalidModel(
                 f"Multiple parameter configurations for {parameter['name']} were found."
             )
         _paramsets_user_configs[parameter.get('name')] = parameter
-
     _reqs = reduce_paramsets_requirements(
         _paramsets_requirements, _paramsets_user_configs
     )
+    return _reqs
 
+
+def _create_parameters_from_spec(_reqs):
     _sets = {}
+    auxdata = []
+    auxdata_order = []
     for param_name, paramset_requirements in _reqs.items():
-        paramset_type = paramset_requirements.get('paramset_type')
+        paramset_type = getattr(pyhf.parameters, paramset_requirements['paramset_type'])
         paramset = paramset_type(**paramset_requirements)
+        if paramset.constrained:  # is constrained
+            auxdata += paramset.auxdata
+            auxdata_order.append(param_name)
         _sets[param_name] = paramset
 
-    return _sets
+    return _sets, auxdata, auxdata_order
 
 
-def _nominal_and_modifiers_from_spec(config, spec):
-    default_data_makers = {
-        'histosys': lambda: {'hi_data': [], 'lo_data': [], 'nom_data': [], 'mask': []},
-        'lumi': lambda: {'mask': []},
-        'normsys': lambda: {'hi': [], 'lo': [], 'nom_data': [], 'mask': []},
-        'normfactor': lambda: {'mask': []},
-        'shapefactor': lambda: {'mask': []},
-        'shapesys': lambda: {'mask': [], 'uncrt': [], 'nom_data': []},
-        'staterror': lambda: {'mask': [], 'uncrt': [], 'nom_data': []},
-    }
+class _nominal_builder:
+    def __init__(self, config):
+        self.mega_samples = {}
+        self.config = config
 
+    def append(self, channel, sample, defined_samp):
+        self.mega_samples.setdefault(sample, {'name': f'mega_{sample}', 'nom': []})
+        nom = (
+            defined_samp['data']
+            if defined_samp
+            else [0.0] * self.config.channel_nbins[channel]
+        )
+        if not len(nom) == self.config.channel_nbins[channel]:
+            raise exceptions.InvalidModel(
+                f'expected {self.config.channel_nbins[channel]} size sample data but got {len(nom)}'
+            )
+        self.mega_samples[sample]['nom'].append(nom)
+
+    def finalize(self):
+        default_backend = pyhf.default_backend
+
+        nominal_rates = default_backend.astensor(
+            [
+                default_backend.concatenate(self.mega_samples[sample]['nom'])
+                for sample in self.config.samples
+            ]
+        )
+        _nominal_rates = default_backend.reshape(
+            nominal_rates,
+            (
+                1,  # modifier dimension.. nominal_rates is the base
+                len(self.config.samples),
+                1,  # alphaset dimension
+                sum(list(self.config.channel_nbins.values())),
+            ),
+        )
+        return _nominal_rates
+
+
+def _nominal_and_modifiers_from_spec(modifier_set, config, spec, batch_size):
     # the mega-channel will consist of mega-samples that subscribe to
     # mega-modifiers. i.e. while in normal histfactory, each sample might
     # be affected by some modifiers and some not, here we change it so that
@@ -115,99 +106,97 @@ def _nominal_and_modifiers_from_spec(config, spec):
     #
     # We don't actually set up the modifier data here for no-ops, but we do
     # set up the entire structure
-    mega_mods = {}
-    for m, mtype in config.modifiers:
-        for s in config.samples:
-            key = f'{mtype}/{m}'
-            mega_mods.setdefault(key, {})[s] = {
-                'type': mtype,
-                'name': m,
-                'data': default_data_makers[mtype](),
-            }
 
-    # helper maps channel-name/sample-name to pairs of channel-sample structs
+    # 1. setup nominal & modifier builders
+    nominal = _nominal_builder(config)
+
+    modifiers_builders = {
+        key: builder(config) for key, (builder, _) in modifier_set.items()
+    }
+
+    # 2. make a helper that maps channel-name/sample-name to pairs of channel-sample structs
     helper = {}
+    _keys_seen = set()
     for c in spec['channels']:
         for s in c['samples']:
-            helper.setdefault(c['name'], {})[s['name']] = (c, s)
+            moddict = {}
+            for x in s['modifiers']:
+                if x['type'] not in modifier_set:
+                    raise exceptions.InvalidModifier(
+                        f'{x["type"]} not among {list(modifier_set.keys())}'
+                    )
+                key = f"{x['type']}/{x['name']}"
+                # check if the modifier to be built is allowed to be shared
+                if not modifiers_builders[x['type']].is_shared and (
+                    key in _keys_seen or key in moddict
+                ):
+                    raise exceptions.InvalidModel(
+                        f"Trying to add paramset {key} on {s['name']} sample in {c['name']} channel but other paramsets exist with the same name."
+                    )
 
-    mega_samples = {}
-    for s in config.samples:
-        mega_nom = []
-        for c in config.channels:
-            defined_samp = helper.get(c, {}).get(s)
-            defined_samp = None if not defined_samp else defined_samp[1]
-            # set nominal to 0 for channel/sample if the pair doesn't exist
-            nom = (
-                defined_samp['data']
-                if defined_samp
-                else [0.0] * config.channel_nbins[c]
-            )
-            mega_nom += nom
-            defined_mods = (
-                {f"{x['type']}/{x['name']}": x for x in defined_samp['modifiers']}
-                if defined_samp
-                else {}
+                moddict[key] = x
+            helper.setdefault(c['name'], {})[s['name']] = (s, moddict)
+            # add in all keys seen
+            _keys_seen.update(moddict)
+
+    # 3. walk spec and call builders
+    for c in config.channels:
+        for s in config.samples:
+            helper_data = helper.get(c, {}).get(s)
+            defined_samp, defined_mods = (
+                (None, None) if not helper_data else helper_data
             )
+            nominal.append(c, s, defined_samp)
             for m, mtype in config.modifiers:
                 key = f'{mtype}/{m}'
                 # this is None if modifier doesn't affect channel/sample.
-                thismod = defined_mods.get(key)
-                # print('key',key,thismod['data'] if thismod else None)
-                if mtype == 'histosys':
-                    lo_data = thismod['data']['lo_data'] if thismod else nom
-                    hi_data = thismod['data']['hi_data'] if thismod else nom
-                    maskval = bool(thismod)
-                    mega_mods[key][s]['data']['lo_data'] += lo_data
-                    mega_mods[key][s]['data']['hi_data'] += hi_data
-                    mega_mods[key][s]['data']['nom_data'] += nom
-                    mega_mods[key][s]['data']['mask'] += [maskval] * len(
-                        nom
-                    )  # broadcasting
-                elif mtype == 'normsys':
-                    maskval = bool(thismod)
-                    lo_factor = thismod['data']['lo'] if thismod else 1.0
-                    hi_factor = thismod['data']['hi'] if thismod else 1.0
-                    mega_mods[key][s]['data']['nom_data'] += [1.0] * len(nom)
-                    mega_mods[key][s]['data']['lo'] += [lo_factor] * len(
-                        nom
-                    )  # broadcasting
-                    mega_mods[key][s]['data']['hi'] += [hi_factor] * len(nom)
-                    mega_mods[key][s]['data']['mask'] += [maskval] * len(
-                        nom
-                    )  # broadcasting
-                elif mtype in ['normfactor', 'shapefactor', 'lumi']:
-                    maskval = bool(thismod)
-                    mega_mods[key][s]['data']['mask'] += [maskval] * len(
-                        nom
-                    )  # broadcasting
-                elif mtype in ['shapesys', 'staterror']:
-                    uncrt = thismod['data'] if thismod else [0.0] * len(nom)
-                    if mtype == 'shapesys':
-                        maskval = [(x > 0 and y > 0) for x, y in zip(uncrt, nom)]
-                    else:
-                        maskval = [bool(thismod)] * len(nom)
-                    mega_mods[key][s]['data']['mask'] += maskval
-                    mega_mods[key][s]['data']['uncrt'] += uncrt
-                    mega_mods[key][s]['data']['nom_data'] += nom
+                thismod = defined_mods.get(key) if defined_mods else None
+                modifiers_builders[mtype].append(key, c, s, thismod, defined_samp)
 
-        sample_dict = {'name': f'mega_{s}', 'nom': mega_nom}
-        mega_samples[s] = sample_dict
+    # 4. finalize nominal & modifier builders
+    nominal_rates = nominal.finalize()
+    finalizd_builder_data = {}
+    for k, (builder, applier) in modifier_set.items():
+        finalizd_builder_data[k] = modifiers_builders[k].finalize()
 
-    nominal_rates = default_backend.astensor(
-        [mega_samples[s]['nom'] for s in config.samples]
+    # 5. collect parameters from spec and from user.
+    # At this point we know all constraints and so forth
+    _required_paramsets = {}
+    for v in list(modifiers_builders.values()):
+        for pname, req_list in v.required_parsets.items():
+            _required_paramsets.setdefault(pname, [])
+            _required_paramsets[pname] += req_list
+
+    user_parameters = spec.get('parameters', [])
+
+    _required_paramsets = _finalize_parameters_specs(
+        user_parameters,
+        _required_paramsets,
     )
-    _nominal_rates = default_backend.reshape(
-        nominal_rates,
-        (
-            1,  # modifier dimension.. nominal_rates is the base
-            len(config.samples),
-            1,  # alphaset dimension
-            sum(list(config.channel_nbins.values())),
-        ),
+    _prameter_objects, _auxdata, _auxdata_order = _create_parameters_from_spec(
+        _required_paramsets
     )
 
-    return mega_mods, _nominal_rates
+    if not _required_paramsets:
+        raise exceptions.InvalidModel('No parameters specified for the Model.')
+
+    config.set_parameters(_prameter_objects)
+    config.set_auxinfo(_auxdata, _auxdata_order)
+
+    # 6. use finalized modifier data to build reparametrization function for main likelihood part
+    the_modifiers = {}
+    for k, (builder, applier) in modifier_set.items():
+        the_modifiers[k] = applier(
+            modifiers=[
+                x for x in config.modifiers if x[1] == k
+            ],  # filter modifier names for that mtype (x[1])
+            pdfconfig=config,
+            builder_data=finalizd_builder_data.get(k),
+            batch_size=batch_size,
+            **config.modifier_settings.get(k, {}),
+        )
+
+    return the_modifiers, nominal_rates
 
 
 class _ModelConfig(_ChannelSummaryMixin):
@@ -228,10 +217,6 @@ class _ModelConfig(_ChannelSummaryMixin):
             spec (:obj:`jsonable`): The HistFactory JSON specification.
         """
         super().__init__(channels=spec['channels'])
-        _required_paramsets = _paramset_requirements_from_modelspec(
-            spec, self.channel_nbins
-        )
-        poi_name = config_kwargs.pop('poi_name', 'mu')
 
         default_modifier_settings = {
             'normsys': {'interpcode': 'code4'},
@@ -247,19 +232,73 @@ class _ModelConfig(_ChannelSummaryMixin):
                 f"Unsupported options were passed in: {list(config_kwargs.keys())}."
             )
 
+        # prefixed with underscore are documented via @property
+        self._par_order = []
+        self._poi_name = None
+        self._poi_index = None
+        self._nmaindata = sum(self.channel_nbins.values())
+        self._auxdata = []
+
+        # these are not documented properties
         self.par_map = {}
-        self.par_order = []
-        self.poi_name = None
-        self.poi_index = None
-        self.auxdata = []
         self.auxdata_order = []
 
-        self._create_and_register_paramsets(_required_paramsets)
-        if poi_name is not None:
-            self.set_poi(poi_name)
+    @property
+    def par_order(self):
+        """
+        Return an ordered list of paramset names in the model.
+        """
+        return self._par_order
 
+    @property
+    def poi_name(self):
+        """
+        Return the name of the POI parameter in the model.
+        """
+        return self._poi_name
+
+    @property
+    def poi_index(self):
+        """
+        Return the index of the POI parameter in the model.
+        """
+        return self._poi_index
+
+    @property
+    def auxdata(self):
+        """
+        Return the auxiliary data in the model.
+        """
+        return self._auxdata
+
+    @property
+    def nmaindata(self):
+        """
+        Return the length of data in the main model.
+        """
+        return self._nmaindata
+
+    @property
+    def nauxdata(self):
+        """
+        Return the length of data in the constraint model.
+        """
+        return len(self._auxdata)
+
+    def set_parameters(self, _required_paramsets):
+        """
+        Evaluate the required parameters for the model configuration.
+        """
+        self._create_and_register_paramsets(_required_paramsets)
         self.npars = len(self.suggested_init())
-        self.nmaindata = sum(self.channel_nbins.values())
+        self.parameters = sorted(k for k in self.par_map.keys())
+
+    def set_auxinfo(self, auxdata, auxdata_order):
+        """
+        Sets a group of configuration data for the constraint terms.
+        """
+        self._auxdata = auxdata
+        self.auxdata_order = auxdata_order
 
     def suggested_init(self):
         """
@@ -319,6 +358,36 @@ class _ModelConfig(_ChannelSummaryMixin):
         """
         return self.par_map[name]['slice']
 
+    @property
+    def par_names(self):
+        """
+        The names of the parameters in the model including binned-parameter indexing.
+
+        Returns:
+            :obj:`list`: Names of the model parameters.
+
+        Example:
+            >>> import pyhf
+            >>> model = pyhf.simplemodels.uncorrelated_background(
+            ...     signal=[12.0, 11.0], bkg=[50.0, 52.0], bkg_uncertainty=[3.0, 7.0]
+            ... )
+            >>> model.config.par_names
+            ['mu', 'uncorr_bkguncrt[0]', 'uncorr_bkguncrt[1]']
+
+        .. versionchanged:: 0.7.0 Changed from method to property attribute.
+        """
+        _names = []
+        for name in self.par_order:
+            param_set = self.param_set(name)
+            if param_set.is_scalar:
+                _names.append(name)
+                continue
+
+            _names.extend(
+                [f'{name}[{index}]' for index in range(param_set.n_parameters)]
+            )
+        return _names
+
     def param_set(self, name):
         """
         The :class:`~pyhf.parameters.paramset` for the model parameter ``name``.
@@ -337,7 +406,7 @@ class _ModelConfig(_ChannelSummaryMixin):
         """
         return self.par_map[name]['paramset']
 
-    def suggested_fixed(self):
+    def suggested_fixed(self) -> List[bool]:
         """
         Identify the fixed parameters in the model.
 
@@ -368,13 +437,15 @@ class _ModelConfig(_ChannelSummaryMixin):
         fixed = []
         for name in self.par_order:
             paramset = self.par_map[name]['paramset']
-            fixed = fixed + [paramset.suggested_fixed] * paramset.n_parameters
+            fixed += paramset.suggested_fixed
         return fixed
 
     def set_poi(self, name):
         """
         Set the model parameter of interest to be model parameter ``name``.
 
+        If ``name`` is ``None``, this will unset the parameter of interest.
+
         Example:
             >>> import pyhf
             >>> model = pyhf.simplemodels.uncorrelated_background(
@@ -384,14 +455,19 @@ class _ModelConfig(_ChannelSummaryMixin):
             >>> model.config.poi_name
             'mu'
         """
-        if name not in [x for x, _ in self.modifiers]:
+        if name is None:
+            self._poi_name = None
+            self._poi_index = None
+            return
+
+        if name not in self.parameters:
             raise exceptions.InvalidModel(
                 f"The parameter of interest '{name:s}' cannot be fit as it is not declared in the model specification."
             )
         s = self.par_slice(name)
         assert s.stop - s.start == 1
-        self.poi_name = name
-        self.poi_index = s.start
+        self._poi_name = name
+        self._poi_index = s.start
 
     def _create_and_register_paramsets(self, required_paramsets):
         next_index = 0
@@ -405,7 +481,7 @@ class _ModelConfig(_ChannelSummaryMixin):
             sl = slice(next_index, next_index + paramset.n_parameters)
             next_index = next_index + paramset.n_parameters
 
-            self.par_order.append(param_name)
+            self._par_order.append(param_name)
             self.par_map[param_name] = {'slice': sl, 'paramset': paramset}
 
 
@@ -496,34 +572,45 @@ class _ConstraintModel:
 class _MainModel:
     """Factory class to create pdfs for the main measurement."""
 
-    def __init__(self, config, mega_mods, nominal_rates, batch_size):
+    def __init__(
+        self,
+        config,
+        modifiers,
+        nominal_rates,
+        batch_size=None,
+        clip_sample_data: Union[float, None] = None,
+        clip_bin_data: Union[float, None] = None,
+    ):
+        default_backend = pyhf.default_backend
+
         self.config = config
-        self._factor_mods = [
-            modtype
-            for modtype, mod in modifiers.uncombined.items()
-            if mod.op_code == 'multiplication'
-        ]
-        self._delta_mods = [
-            modtype
-            for modtype, mod in modifiers.uncombined.items()
-            if mod.op_code == 'addition'
-        ]
+
+        self._factor_mods = []
+        self._delta_mods = []
         self.batch_size = batch_size
 
+        self.clip_sample_data = clip_sample_data
+        self.clip_bin_data = clip_bin_data
+
+        if self.clip_sample_data is not None:
+            log.warning(
+                f"Clipping expected data per-bin for each sample below {self.clip_sample_data}"
+            )
+
+        if self.clip_bin_data is not None:
+            log.warning(f"Clipping expected data per-bin below {self.clip_bin_data}")
+
         self._nominal_rates = default_backend.tile(
             nominal_rates, (1, 1, self.batch_size or 1, 1)
         )
 
-        self.modifiers_appliers = {
-            k: c(
-                [x for x in config.modifiers if x[1] == k],  # x[1] is mtype
-                config,
-                mega_mods,
-                batch_size=self.batch_size,
-                **config.modifier_settings.get(k, {}),
-            )
-            for k, c in modifiers.combined.items()
-        }
+        self.modifiers_appliers = modifiers
+
+        for modifier_applier in self.modifiers_appliers.values():
+            if modifier_applier.op_code == "addition":
+                self._delta_mods.append(modifier_applier.name)
+            elif modifier_applier.op_code == "multiplication":
+                self._factor_mods.append(modifier_applier.name)
 
         self._precompute()
         events.subscribe('tensorlib_changed')(self._precompute)
@@ -551,7 +638,7 @@ class _MainModel:
         Compute the logarithm of the value of the probability density.
 
         Args:
-            maindata (:obj:`tensor`): The main channnel data (a subset of the full data in a HistFactory model)
+            maindata (:obj:`tensor`): The main channel data (a subset of the full data in a HistFactory model)
             pars (:obj:`tensor`): The model parameters
 
         Returns:
@@ -617,6 +704,11 @@ class _MainModel:
         allfac = tensorlib.concatenate(factors + [nom_plus_delta])
 
         newbysample = tensorlib.product(allfac, axis=0)
+        if self.clip_sample_data is not None:
+            newbysample = tensorlib.clip(
+                newbysample, self.clip_sample_data, max_value=None
+            )
+
         if return_by_sample:
             batch_first = tensorlib.einsum('ij...->ji...', newbysample)
             if self.batch_size is None:
@@ -624,6 +716,9 @@ class _MainModel:
             return batch_first
 
         newresults = tensorlib.sum(newbysample, axis=0)
+        if self.clip_bin_data is not None:
+            newresults = tensorlib.clip(newresults, self.clip_bin_data, max_value=None)
+
         if self.batch_size is None:
             return newresults[0]
         return newresults
@@ -632,49 +727,65 @@ class _MainModel:
 class Model:
     """The main pyhf model class."""
 
-    def __init__(self, spec, batch_size=None, **config_kwargs):
+    def __init__(
+        self,
+        spec,
+        modifier_set=None,
+        batch_size=None,
+        validate: bool = True,
+        clip_sample_data: Union[float, None] = None,
+        clip_bin_data: Union[float, None] = None,
+        **config_kwargs,
+    ):
         """
         Construct a HistFactory Model.
 
         Args:
             spec (:obj:`jsonable`): The HistFactory JSON specification
-            batch_size (:obj:`None` or :obj:`int`): Number of simultaneous (batched) Models to compute.
+            batch_size (:obj:`None` or :obj:`int`): Number of simultaneous (batched)
+             Models to compute.
+            validate (:obj:`bool`): Whether to validate against a JSON schema
+            clip_sample_data (:obj:`None` or :obj:`float`): Clip the minimum value of expected data by-sample. Default is no clipping.
+            clip_bin_data (:obj:`None` or :obj:`float`): Clip the minimum value of expected data by-bin. Default is no clipping.
             config_kwargs: Possible keyword arguments for the model configuration
 
         Returns:
             model (:class:`~pyhf.pdf.Model`): The Model instance.
 
         """
+        modifier_set = modifier_set or histfactory_set
+
         self.batch_size = batch_size
         # deep-copy "spec" as it may be modified by config
         self.spec = copy.deepcopy(spec)
         self.schema = config_kwargs.pop('schema', 'model.json')
         self.version = config_kwargs.pop('version', None)
         # run jsonschema validation of input specification against the (provided) schema
-        log.info(f"Validating spec against schema: {self.schema:s}")
-        utils.validate(self.spec, self.schema, version=self.version)
+        if validate:
+            log.info(f"Validating spec against schema: {self.schema:s}")
+            schema.validate(self.spec, self.schema, version=self.version)
         # build up our representation of the specification
-        self.config = _ModelConfig(spec, **config_kwargs)
+        poi_name = config_kwargs.pop('poi_name', 'mu')
+        self._config = _ModelConfig(self.spec, **config_kwargs)
 
-        mega_mods, _nominal_rates = _nominal_and_modifiers_from_spec(self.config, spec)
-        self.main_model = _MainModel(
-            self.config,
-            mega_mods=mega_mods,
-            nominal_rates=_nominal_rates,
-            batch_size=self.batch_size,
+        modifiers, _nominal_rates = _nominal_and_modifiers_from_spec(
+            modifier_set, self.config, self.spec, self.batch_size
         )
 
-        # this is tricky, must happen before constraint
-        # terms try to access auxdata but after
-        # combined mods have been created that
-        # set the aux data
-        for k in sorted(self.config.par_map.keys()):
-            parset = self.config.param_set(k)
-            if hasattr(parset, 'pdf_type'):  # is constrained
-                self.config.auxdata += parset.auxdata
-                self.config.auxdata_order.append(k)
-        self.config.nauxdata = len(self.config.auxdata)
+        poi_name = None if poi_name == "" else poi_name
+        if poi_name is not None:
+            self.config.set_poi(poi_name)
 
+        self.main_model = _MainModel(
+            self.config,
+            modifiers=modifiers,
+            nominal_rates=_nominal_rates,
+            batch_size=self.batch_size,
+            clip_sample_data=clip_sample_data,
+            clip_bin_data=clip_bin_data,
+        )
+
+        # the below call needs auxdata order for example
         self.constraint_model = _ConstraintModel(
             config=self.config, batch_size=self.batch_size
         )
@@ -688,6 +799,13 @@ class Model:
             sizes, ['main', 'aux'], self.batch_size
         )
 
+    @property
+    def config(self):
+        """
+        The :class:`_ModelConfig` instance for the model.
+        """
+        return self._config
+
     def expected_auxdata(self, pars):
         """
         Compute the expected value of the auxiliary measurements.
@@ -782,8 +900,6 @@ class Model:
             pdf: A distribution object implementing the main measurement pdf of HistFactory
 
         """
-        tensorlib, _ = get_backend()
-
         pdfobjs = []
         mainpdf = self.main_model.make_pdf(pars)
         if mainpdf:
